#### Example 3.1 - Additive Model Example ####

library(huge)  
library(quantreg)
library(splines)
source("MVSIS.R") #source code of MVSIS

#we set (n,p)=(2000,200) and simulate 100 datasets, only the first 4 predictors are relevant to the response variable. We use ditrue to store
#store the scores produced by method i for each relevant predictor in each dataset, and use difalse to store the scores produced by method i for each
#irrelevant predictor in each dataset.

p<-2000
n<-200

d1true<-matrix(0,100,4)
d2true<-matrix(0,100,4)
d3true<-matrix(0,100,4)
d4true<-matrix(0,100,4)
d5true<-matrix(0,100,4)
d6true<-matrix(0,100,4)

d1false<-matrix(0,100,p-4)
d2false<-matrix(0,100,p-4)
d3false<-matrix(0,100,p-4)
d4false<-matrix(0,100,p-4)
d5false<-matrix(0,100,p-4)
d6false<-matrix(0,100,p-4)


for (iter in 1:100)
{
#Case 1 we use t(4) distributiont to generate predictors
 X<-matrix(rt(n*p,4),n,p)
 response<- -3*sin(2*X[,1])+(X[,2]*X[,2]-25.0/12)-1.5*X[,3]+(exp(-X[,4])-2.0/5*sinh(5/2))+rt(n,1)
 
#Case 2 uniform(-2.5,2.5) distribution to generate predictors
 #X<-matrix(runif(n*p,-2.5,2.5),n,p)
 #response<- -3*sin(2*X[,1])+(X[,2]*X[,2]-25.0/12)-1.5*X[,3]+(exp(-X[,4])-2.0/5*sinh(5/2))+rt(n,1)
 
 ##Method1  MV-SIS
 
 mu<-NULL
 for(k in 1:p){
   q<-quantile(X[,k],c(0.25,0.5,0.75))
   R<-X[,k]
   R[X[,k]<q[1]]<-1
   R[X[,k]>=q[1] & X[,k]<q[2]]<-2
   R[X[,k]>=q[2] & X[,k]<q[3]]<-3
   R[X[,k]>=q[3]]<-4
   u<-MV(response,R)
   mu<-c(mu,u)
 }
 
 #Method 2 HZ-SIS
 
 #Nonparanormal transformation
 Z<-cbind(response,X)
 Z<-huge.npn(Z,npn.func='truncation')
 y<-Z[,1]
 Z<-Z[,2:(p+1)]
 
 #Calculate HZ statistic
 HZ<-1:p
 for (k in 1:p){
   beta<-1/sqrt(2)*(n*(2*2+1)/4)^(1/(2+4))
   d<-cbind(y,Z[,k]) 
   diffy<-matrix(rep(y,n),n,n,byrow=TRUE)-matrix(rep(y,n),n,n,byrow=FALSE)
   diffx<-matrix(rep(Z[,k],n),n,n,byrow=TRUE)-matrix(rep(Z[,k],n),n,n,byrow=FALSE)
   Dij<-diffy^2+diffx^2
   Di<-diag((d)%*%t(d))
   HZ[k]<-sum(exp(-beta*beta/2*Dij))/n-2*(1+beta^2)^(-2/2)*sum(exp(-beta^2/(2*(1+beta^2))*Di))+n*(1+2*beta^2)^(-2/2)
  }
 

 #Method 3  ##DC-SIS
 DCcorr<-1:p
 Mresponse<-matrix(rep(response,n),n,n,byrow=TRUE)-matrix(rep(response,n),n,n,byrow=FALSE)  
 S1<-mean(Mresponse^2)  
 S2<-(mean(abs(Mresponse)))^2
 S3<-mean((apply(abs(Mresponse),2,mean))^2)
 DCcovy<-sqrt(S1+S2-2*S3)
 for (k in 1:p){
   Mx<-matrix(rep(X[,k],n),n,byrow=TRUE)-matrix(rep(X[,k],n),n,byrow=FALSE)
   S1<-mean(Mx^2)  
   S2<-(mean(abs(Mx)))^2
   S3<-mean((apply(abs(Mx),2,mean))^2)
   DCcovx<-sqrt(S1+S2-2*S3)
   S1<-mean(abs(Mresponse)*abs(Mx))
   S2<-mean(abs(Mresponse))*mean(abs(Mx))  
   S3<-mean((apply(abs(Mresponse),2,mean))*(apply(abs(Mx),2,mean)))
   DCcovxy<-sqrt(S1+S2-2*S3)
   DCcorr[k]<-DCcovxy/sqrt(DCcovy*DCcovx)
 }
 
 #method 4 Qa-SIS
 Qasis<-1:p
 for (k in 1:p)
 {
   xx<- model.matrix(response ~ bs(X[,k], df=3))
   fit <- rq(response ~ bs(X[,k], df=3), tau=0.5, data=data.frame(X[,k],response))
   accel.fit <- xx %*% fit$coef
   Qasis[k]<-mean((accel.fit-median(response))^2)
 } 
 
 
 #method 5 SIS
 sis<-1:p
 for (k in 1:p)
 {sis[k]=abs(cor(X[,k],y))}
 
 #method 6 NIS
 NIS<-1:p
 for (k in 1:p)
 {
   xx<- model.matrix(response ~ bs(X[,k], df=3))
   fit <- lm(response ~ bs(X[,k], df=3))
   accel.fit <- xx %*% fit$coef
   NIS[k]<-mean((accel.fit-mean(response))^2)
 } 
 
 
 #Store the Result
 d1<-cbind(1:p,mu)
 d2<-cbind(1:p,HZ)
 d3<-cbind(1:p,DCcorr)
 d4<-cbind(1:p,Qasis)
 d5<-cbind(1:p,sis)
 d6<-cbind(1:p,NIS)
 
 print(iter)
 
 d1true[iter,]=d1[1:4,2]
 d2true[iter,]=d2[1:4,2]
 d3true[iter,]=d3[1:4,2]
 d4true[iter,]=d4[1:4,2]
 d5true[iter,]=d5[1:4,2]
 d6true[iter,]=d6[1:4,2]
 
 d1false[iter,]=d1[-c(1:4),2]
 d2false[iter,]=d2[-c(1:4),2]
 d3false[iter,]=d3[-c(1:4),2]
 d4false[iter,]=d4[-c(1:4),2]
 d5false[iter,]=d5[-c(1:4),2]
 d6false[iter,]=d6[-c(1:4),2]

 }
 
d1full<-cbind(d1true,d1false)
d2full<-cbind(d2true,d2false)
d3full<-cbind(d3true,d3false)
d4full<-cbind(d4true,d4false)
d5full<-cbind(d5true,d5false)
d6full<-cbind(d6true,d6false)

d1rank<-matrix(0,100,p)
d2rank<-matrix(0,100,p)
d3rank<-matrix(0,100,p)
d4rank<-matrix(0,100,p)
d5rank<-matrix(0,100,p)
d6rank<-matrix(0,100,p)

for (iter in 1:100)
{
  d1rank[iter,]<-p+1-rank(d1full[iter,])
  d2rank[iter,]<-p+1-rank(d2full[iter,])
  d3rank[iter,]<-p+1-rank(d3full[iter,])
  d4rank[iter,]<-p+1-rank(d4full[iter,])
  d5rank[iter,]<-p+1-rank(d5full[iter,])
  d6rank[iter,]<-p+1-rank(d6full[iter,])
}

MSD<-function(x)
{
  print(max(x[1:4]))
}

# Output: MSD for each method in each simulation
d1result<-apply(d1rank,1,MSD)
d2result<-apply(d2rank,1,MSD)
d3result<-apply(d3rank,1,MSD)
d4result<-apply(d4rank,1,MSD)
d5result<-apply(d5rank,1,MSD)
d6result<-apply(d6rank,1,MSD)

